@@ -24,10 +24,10 @@
  
 int nextSong(int checkFlags);
 int prevSong(int checkFlags);
-void play(GtkWidget *widget, configType *options);
+int play(GtkWidget *widget, configType *options);
 void stop();
-void back(GtkWidget *widget, configType *options);
-void forward(GtkWidget *widget, configType *options);
+int back(GtkWidget *widget, configType *options);
+int forward(GtkWidget *widget, configType *options);
 void pauseSong();
 
-#endif
\ No newline at end of file
+#endif
