@@ -23,6 +23,8 @@
 #include <unistd.h>
 #include <signal.h>
 #include <strings.h>
+#include <string.h>
+#include <stdlib.h>
 
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
@@ -62,6 +64,24 @@ long int startSecs = 0;
 long int currSecs = 0;
 int oldSeconds = 0;
 
+#define TOKENCOUNT 16
+tokenInfo tokens[TOKENCOUNT] = { { "buffer_size", INTEGER, NULL },
+                                 { "downsample", INTEGER, NULL },
+                                 { "output_type", INTEGER, NULL },
+                                 { "send_to_lineout", BOOLEAN, NULL },
+                                 { "managed", BOOLEAN, NULL },
+                                 { "tooltips", BOOLEAN, NULL },
+                                 { "autoload", STRING, NULL },
+                                 { "autoparse", BOOLEAN, NULL},
+                                 { "mixerapp", STRING, NULL },
+                                 { "themepack", STRING, NULL },
+                                 { "autoloadtags", BOOLEAN, NULL},
+                                 { "mp3loaddir", STRING, NULL},
+                                 { "mp3player", STRING, NULL},
+                                 { "sound_device", STRING, NULL},
+                                 { "http_proxy", STRING, NULL},
+                                 { "http_auth_string", STRING, NULL} };
+
 void bringToFront(GtkWidget *widget, GdkEventVisibility *event);
 void createSmallWindow();
 void createBigWindow();
@@ -70,6 +90,10 @@ void installSigHandler();
 
 /********** Main Guts ***********/
 
+/***********************
+ * Signal Handler Code *
+ ***********************/
+
 void childKill(int signal)
 {
   int status;      
@@ -93,6 +117,22 @@ void installSigHandler()
   signal(SIGCHLD, childKill);
 }
 
+void openDialog(GtkWidget *widget, GtkWidget *dialog)
+{
+  if (! GTK_WIDGET_VISIBLE(dialog))
+    gtk_widget_show(dialog);
+}
+
+void closeDialog(GtkWidget *widget, GtkWidget *dialog)
+{
+  if (GTK_WIDGET_VISIBLE(dialog))
+    gtk_widget_hide(dialog);
+}
+
+/********************************
+ * RC File Loading/Saving Stuff *
+ ********************************/
+
 void adjustThemePath()
 {
   char *homeDir = getenv("HOME");
@@ -121,46 +161,55 @@ void adjustThemePath()
   free(fullFile);
 }
 
-#define TOKENCOUNT 11
+gint checkMP3Player()
+{
+  if (access(options.mp3player, F_OK) < 0)
+    {
+      fprintf(stderr, "Error, %s not found!\n", options.mp3player);
+      return 0;
+    }
+  else
+    return 1;
+}
 
-void loadRCFile()
-{  
-  tokenInfo tokens[TOKENCOUNT] = { { "buffer_size", INTEGER, NULL },
-                                   { "downsample", INTEGER, NULL },
-                                   { "output_type", INTEGER, NULL },
-                                   { "send_to_lineout", BOOLEAN, NULL },
-                                   { "managed", BOOLEAN, NULL },
-                                   { "tooltips", BOOLEAN, NULL },
-                                   { "autoload", STRING, NULL },
-                                   { "autoparse", BOOLEAN, NULL},
-                                   { "mixerapp", STRING, NULL },
-                                   { "themepack", STRING, NULL },
-                                   { "autoloadtags", BOOLEAN, NULL} };
-                                                             
+void setupRCTokens()
+{
   char *homeDir = getenv("HOME");
-  char *rcFile = (char *)malloc(strlen(homeDir)+strlen(RCFILENAME)+
-                                strlen(RCFILEDIR)+4);
-  
-  strcpy(rcFile, homeDir);
-  strcat(rcFile, RCFILEDIR);
-  strcat(rcFile, "/");
-  strcat(rcFile, RCFILENAME);
-  
+
   options.bufferSize = 1500;
   options.downSample = 0;              /* Create default settings. */
   options.output = 2;
   options.lineOut = FALSE;
   options.managed = TRUE;
   options.toolTips = TRUE;
-  options.playListName = (char *)malloc(255);
-  options.playListName[0] = '\0';
-  options.autoparse = FALSE;  
-  options.mixerApp = (char *)malloc(255);
-  options.mixerApp[0] = '\0';
-  options.themePack = (char *)malloc(1024);
+  options.autoparse = FALSE;      
   options.autoloadTags = FALSE;
-  strcpy(options.themePack, "default");  
+  
+  options.playListName = (char *)malloc(1024);
+  options.playListName[0] = '\0';  
+  
+  options.mixerApp = (char *)malloc(1024);
+  options.mixerApp[0] = '\0';
 
+  options.themePack = (char *)malloc(1024);  
+  strcpy(options.themePack, "default");
+
+  options.mp3loaddir = (char *)malloc(2048);
+  strncpy(options.mp3loaddir, homeDir, 1023);
+  strcat(options.mp3loaddir, "/");
+  
+  options.mp3player = (char *)malloc(1024);
+  strncpy(options.mp3player, MPG123_NAME, 1023);  
+  
+  options.soundDevice = (char *)malloc(1024);
+  strncpy(options.soundDevice, SOUND_DEV, 1023);
+  
+  options.httpProxy = (char *)malloc(1024);
+  strncpy(options.httpProxy, HTTP_PROXY, 1023);
+  
+  options.httpAuthString = (char *)malloc(1024);
+  strncpy(options.httpAuthString, HTTP_AUTH_STRING, 1023);
+  
   tokens[0].dataBuffer = &options.bufferSize;
   tokens[1].dataBuffer = &options.downSample;
   tokens[2].dataBuffer = &options.output;     /* Set up the tokens correctly */
@@ -172,12 +221,29 @@ void loadRCFile()
   tokens[8].dataBuffer = options.mixerApp;
   tokens[9].dataBuffer = options.themePack;
   tokens[10].dataBuffer = &options.autoloadTags;
-  
+  tokens[11].dataBuffer = options.mp3loaddir;
+  tokens[12].dataBuffer = options.mp3player;
+  tokens[13].dataBuffer = options.soundDevice;
+  tokens[14].dataBuffer = options.httpProxy;
+  tokens[15].dataBuffer = options.httpAuthString;
+}
+
+void loadRCFile()
+{                                                                 
+  char *homeDir = getenv("HOME");
+  char *rcFile = (char *)malloc(strlen(homeDir)+strlen(RCFILENAME)+
+                                strlen(RCFILEDIR)+4); 
+  strcpy(rcFile, homeDir);
+  strcat(rcFile, RCFILEDIR);
+  strcat(rcFile, "/");
+  strcat(rcFile, RCFILENAME);
+
   parseFile(rcFile, tokens, TOKENCOUNT);
 
   if (options.playListName[0] != '\0')  /* Read playlist from autoload param */
-    loadAlbumns(options.playListName);
+      loadAlbumns(options.playListName);
   
+  checkMP3Player();
   adjustThemePath();
   
   free(rcFile);
@@ -185,7 +251,6 @@ void loadRCFile()
 
 void saveRCFile()
 {
-  FILE *outputFile;
   char *homeDir = getenv("HOME");
   char *rcFile = (char *)malloc(strlen(homeDir)+strlen(RCFILENAME)+
                                 strlen(RCFILEDIR)+4);
@@ -201,50 +266,7 @@ void saveRCFile()
   if (access(rcDir, F_OK) < 0)
     mkdir(rcDir, 0755);
   
-  outputFile = fopen(rcFile, "w");
-  
-  if (outputFile)
-    {  
-      fprintf(outputFile, "buffer_size = %d\n", options.bufferSize);
-      fprintf(outputFile, "downsample = %d\n", options.downSample);
-      fprintf(outputFile, "output_type = %d\n", options.output);
-  
-      if (options.lineOut)
-        fprintf(outputFile, "send_to_lineout = TRUE\n");
-      else
-        fprintf(outputFile, "send_to_lineout = FALSE\n");
-  
-      if (options.managed)
-        fprintf(outputFile, "managed = TRUE\n");
-      else
-        fprintf(outputFile, "managed = FALSE\n");
-    
-      
-      if (options.toolTips)
-        fprintf(outputFile, "tooltips = TRUE\n");
-      else
-        fprintf(outputFile, "tooltips = FALSE\n");  
-        
-      if (options.autoparse)
-        fprintf(outputFile, "autoparse = TRUE\n");
-      else
-        fprintf(outputFile, "autoparse = FALSE\n");    
-        
-      if (options.autoloadTags)
-        fprintf(outputFile, "autoloadtags = TRUE\n");
-      else
-        fprintf(outputFile, "autoloadtags = FALSE\n");    
-        
-      if (options.playListName[0] != '\0')
-        fprintf(outputFile, "autoload = %s\n", options.playListName);
-
-      if (options.mixerApp[0] != '\0')
-        fprintf(outputFile, "mixerapp = %s\n", options.mixerApp);
-        
-      fprintf(outputFile, "themepack = %s\n", options.themePack);
-              
-      fclose(outputFile);    
-    }  
+  saveFile(rcFile, tokens, TOKENCOUNT);
 }
 
 void loadCommandlineFiles(int argc, char *argv[])
@@ -260,10 +282,59 @@ void loadCommandlineFiles(int argc, char *argv[])
     }
 }
 
+/*********************************
+ * Callback Functions for Events *
+ *********************************/
+
+void destroyWindow(GtkWidget *widget, GtkWidget *window)
+{
+  gtk_widget_destroy(window);
+}
+
+void missingFileError()
+{
+  GtkWidget *window;
+  GtkWidget *label;
+  GtkWidget *button;
+  GtkWidget *vbox, *hbox;
+  char *errorStr;
+  char *filename = ((songNode *)(currentSong->data))->fileName;
+  
+  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title(GTK_WINDOW(window), "Play Error");
+  gtk_window_set_wmclass(GTK_WINDOW(window), "gmp3.error_dialog", NULL);
+
+  errorStr = (char *)malloc(strlen("Error Opening File: ")+
+                            strlen(filename)+4);
+  sprintf(errorStr, "Error Opening File: %s", filename);
+  
+  hbox = gtk_hbox_new(FALSE, 5);
+  gtk_container_add(GTK_CONTAINER(window), hbox);
+  gtk_widget_show(hbox);
+  
+  vbox = gtk_vbox_new(FALSE, 5);
+  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 5);
+  gtk_widget_show(vbox);
+
+  label = gtk_label_new(errorStr);
+  gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 5);
+  gtk_widget_show(label);
+  
+  button = gtk_button_new_with_label("Close");
+  gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 5);
+  gtk_widget_show(button);  
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(destroyWindow), window);
+
+  gtk_widget_show(window);
+}
+
 void printSongName()
 {          
   char *currentSongName;
-  
+  char *currentFileName;
+  gint i;
+
   if (activeWindow->song_label.eventBox == NULL)
     return;
 
@@ -271,7 +342,20 @@ void printSongName()
     return;
 
   currentSongName = ((songNode *)(currentSong->data))->title;
-  gtk_label_set(GTK_LABEL(activeWindow->song_label.widget->widget), currentSongName);
+  currentFileName = ((songNode *)(currentSong->data))->fileName;
+
+  /* get rid of path in filenames [pel] */
+  if (strcmp(currentSongName, currentFileName) == 0)
+    {
+      i = strlen(currentSongName);
+  
+      for (i = strlen(currentSongName); currentSongName[i] != '/'; i--);
+      i++;
+  
+      currentSongName += i;
+    }
+
+  gtk_label_set(GTK_LABEL(activeWindow->song_label.widget->widget), currentSongName); 
 }
 
 void resetClock()
@@ -288,7 +372,7 @@ int playLoop()
     {    
       if (! nextSong(TRUE))
         {
-          play(NULL, &options);
+          if (! play(NULL, &options)) missingFileError();
 
           playing = TRUE;
           doneSong = FALSE;
@@ -359,7 +443,7 @@ int adjustTimer()
 void stopCallback()
 {
   if (paused && activeWindow->pause_label.eventBox)
-    gtk_widget_hide(activeWindow->pause_label.eventBox);
+    gtk_widget_hide(activeWindow->pause_label.widget->widget);
     
   stop();  
 }
@@ -376,7 +460,18 @@ void playCallback()
     }
 
   resetClock();
-  play(NULL, &options);    
+  
+  if (! play(NULL, &options)) missingFileError();
+}
+
+void backCallback()
+{
+  if (! back(NULL, &options)) missingFileError();
+}
+
+void forwardCallback()
+{
+  if (! forward(NULL, &options)) missingFileError();
 }
 
 void togglePause()
@@ -438,16 +533,70 @@ void toggleLoop()
 }
 
 void quitCallback()
-{
+{ 
+  static GtkWidget *window = NULL;
+  GtkWidget *hbox, *hbox2, *vbox;
+  GtkWidget *button;
+  GtkWidget *label;
+
+  if (window && GTK_WIDGET_VISIBLE(window))
+    return;
+
   if ((playing) && (! doneSong))
     stop();
-   
-  gtk_main_quit();
-}
 
-void destroy_window(GtkWidget *widget, GtkWidget **window)
-{  
-  *window = NULL;
+  if (albumnChanged && ! window)
+    {
+      window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+      gtk_window_set_title(GTK_WINDOW(window), "Alert!");
+      gtk_window_set_wmclass(GTK_WINDOW(window), "gmp3.alert", NULL);
+      
+      gtk_signal_connect(GTK_OBJECT(window), "delete_event", 
+                         GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
+                         
+      gtk_signal_connect(GTK_OBJECT(window), "destroy_event",
+                         GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
+      
+      vbox = gtk_vbox_new(FALSE, 5);
+      gtk_container_add(GTK_CONTAINER(window), vbox);
+      gtk_widget_show(vbox);
+
+      hbox = gtk_hbox_new(FALSE, 5);
+      gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);
+      gtk_widget_show(hbox);
+      
+      label = gtk_label_new("You haven't saved your albumns!");
+      gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 5);
+      gtk_widget_show(label);
+      
+      hbox2 = gtk_hbox_new(FALSE, 5);
+      gtk_box_pack_start(GTK_BOX(vbox), hbox2, TRUE, TRUE, 5);
+      gtk_widget_show(hbox2);
+      
+      hbox = gtk_hbox_new(FALSE, 0);
+      gtk_box_pack_start(GTK_BOX(hbox2), hbox, TRUE, TRUE, 5);
+      gtk_widget_show(hbox);
+      
+      button = gtk_button_new_with_label("Quit");
+      gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+      gtk_widget_show(button);
+      
+      gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                         GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+      
+      button = gtk_button_new_with_label("Cancel");
+      gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+      gtk_widget_show(button);      
+      
+      gtk_signal_connect(GTK_OBJECT(button), "clicked", 
+                         GTK_SIGNAL_FUNC(closeDialog), window);
+      
+      gtk_widget_show(window);
+    }
+  else if (albumnChanged)
+    gtk_widget_show(window);
+  else
+    gtk_main_quit();
 }
 
 void volumeSliderCallback(GtkAdjustment *adjustment, gpointer *data)
@@ -522,14 +671,15 @@ void widgetCommandCallback(GtkWidget *widget, gint command)
   switch (command)
     {
     case CMD_NO_COMMAND : { break; }
-    case CMD_BACK : { back(NULL, &options); break; }
-    case CMD_FORWARD : { forward(NULL, &options); break; }
+    case CMD_BACK : { backCallback(NULL, &options); break; }
+    case CMD_FORWARD : { forwardCallback(NULL, &options); break; }
     case CMD_PLAY : { playCallback(NULL, &options); break; }
     case CMD_STOP : { stopCallback(NULL, &options); break; }
     case CMD_PAUSE : { togglePause(NULL, &options); break; }
     case CMD_TOGGLE_SHUFFLE : { toggleShuffle(NULL, &options); break; }
     case CMD_TOGGLE_REPEAT : { toggleLoop(NULL, &options); break; }      
-    case CMD_OPEN_PLAYLIST_EDITOR : { createListDialog(); break; }
+    case CMD_OPEN_ALBUMN_EDITOR : { openAlbumnEditorDialog(); break; }
+    case CMD_OPEN_PLAYLIST_EDITOR : { openPlaylistEditorDialog(); break; }
     case CMD_OPEN_OPTIONS : { createOptionsDialog(); break; }
     case CMD_OPEN_ABOUT_BOX : { createAboutBox(); break; }
     case CMD_OPEN_MIXER : { openMixer(); break; }      
@@ -546,7 +696,7 @@ void widgetCommandCallback(GtkWidget *widget, gint command)
 gint widgetLClickCommandCallback(GtkWidget *widget, GdkEventButton *event,
                                  gint command)
 {
-  if ((event->type != GDK_BUTTON_PRESS) || (event->button != 1))
+  if (event->button != 1)
     return FALSE;
 
   switch (command)
@@ -556,20 +706,18 @@ gint widgetLClickCommandCallback(GtkWidget *widget, GdkEventButton *event,
         gtk_menu_popup(GTK_MENU(activeWindow->main_menu),
                        NULL, NULL, NULL, NULL,
                        3, event->time);
-        break;
+        return TRUE;
       }  
-    case CMD_NO_COMMAND :
-      return FALSE;  
-    default : { widgetCommandCallback(widget, command); break; }
+    case CMD_DRAG_CAPTURE : shape_pressed(widget, event, activeWindow);
+    case CMD_NO_COMMAND : return FALSE;
+    default : { widgetCommandCallback(widget, command); return TRUE; }
     }
-    
-  return TRUE;
 }                                 
 
 gint widgetMClickCommandCallback(GtkWidget *widget, GdkEventButton *event,
                                  gint command)
 {
-  if ((event->type != GDK_BUTTON_PRESS) || (event->button != 2))
+  if (event->button != 2)
     return FALSE;
 
   switch (command)
@@ -581,6 +729,7 @@ gint widgetMClickCommandCallback(GtkWidget *widget, GdkEventButton *event,
                        3, event->time);
         break;
       }  
+    case CMD_DRAG_CAPTURE : shape_pressed(widget, event, activeWindow);
     case CMD_NO_COMMAND :
       return FALSE;        
     default : { widgetCommandCallback(widget, command); break; }
@@ -592,7 +741,7 @@ gint widgetMClickCommandCallback(GtkWidget *widget, GdkEventButton *event,
 gint widgetRClickCommandCallback(GtkWidget *widget, GdkEventButton *event,
                                  gint command)
 {
-  if ((event->type != GDK_BUTTON_PRESS) || (event->button != 3))
+  if (event->button != 3)
     return FALSE;
 
   switch (command)
@@ -604,6 +753,7 @@ gint widgetRClickCommandCallback(GtkWidget *widget, GdkEventButton *event,
                        3, event->time);
         break;
       }  
+    case CMD_DRAG_CAPTURE : shape_pressed(widget, event, activeWindow);
     case CMD_NO_COMMAND :
       return FALSE;
     default : { widgetCommandCallback(widget, command); break; }
@@ -612,6 +762,10 @@ gint widgetRClickCommandCallback(GtkWidget *widget, GdkEventButton *event,
   return TRUE;
 }                                 
 
+/***********************
+ * Main initialization *
+ ***********************/
+
 int main(int argc, char *argv[])
 {
   installSigHandler();
@@ -620,12 +774,14 @@ int main(int argc, char *argv[])
   gdk_imlib_init();
   root_win = gdk_window_foreign_new(GDK_ROOT_WINDOW());    
 
-  g_print("GMP3 Version %s\n", VERSION);   
+  g_print("GMP3 Version %s\n", VERSION);     
   initializeMixer();
   initializePlaylist();  
   initializeAlbumnList(&albumnList);
 
+  setupRCTokens();
   loadRCFile();
+  
   parseThemeRC(&bigWindow,
                &smallWindow);
   activeWindow = &bigWindow;
@@ -635,9 +791,12 @@ int main(int argc, char *argv[])
   gtk_timeout_add(1000, (GtkFunction) adjustTimer, NULL);
   adjustTimer();
   
-  gtk_timeout_add(100, (GtkFunction) playLoop, NULL);
+  gtk_timeout_add(100, (GtkFunction) playLoop, NULL);    
+  
   gtk_main();  
   
+  saveRCFile();
+  
   destroyPlaylist();
   destroyAlbumnList(albumnList);
   closeMixer();
