@@ -24,6 +24,7 @@
 #include "list.h"
 #include "handlers.h"
 #include "listutils.h"
+#include "gmp3.h"               /* for the configType declaration */
 
 const char *albumnlist_id_string = "*gmp3_albumnlist*";
 const char *playlist_id_string = "*gmp3_playlist_2*";
@@ -43,6 +44,11 @@ GtkWidget *treeMenu;
 GtkWidget *songMenu;
 GtkWidget *playlistMenu;
 
+GtkWidget *editorWindow = NULL;
+GtkWidget *playlistWindow = NULL;  
+
+int albumnChanged = FALSE;
+
 /************************
   Basic Utility Routines
  ************************/
@@ -53,32 +59,34 @@ void chop(char *string)
     string[strlen(string)-1] = '\0';
 }
 
-void openDialog(GtkWidget *widget, GtkWidget *dialog)
-{
-  if (! GTK_WIDGET_VISIBLE(dialog))
-    gtk_widget_show(dialog);
-}
-
-void closeDialog(GtkWidget *widget, GtkWidget *dialog)
-{
-  if (GTK_WIDGET_VISIBLE(dialog))
-    gtk_widget_hide(dialog);
-}
-
 char *completeFileName(GtkFileSelection *fileSel, char *fileName)
 {
   CompletionState *cmpl_state;
-  char *fullFileName;
-  
+  char *fullFileName, *fullDirName;
+  extern configType options;
+  gint i, len;
+
   cmpl_state = fileSel->cmpl_state;
   
   fullFileName = (char *)malloc(strlen(fileName)+
                                 strlen(cmpl_state->reference_dir->fullname)+2);
   
-  strcpy(fullFileName, cmpl_state->reference_dir->fullname);;
+  strcpy(fullFileName, cmpl_state->reference_dir->fullname);
   strcat(fullFileName, "/");
   strcat(fullFileName, fileName);
   
+/* save the dir name of the last mp3 loaded [pel] cleaned up by Brett */
+  len = strlen(fullFileName);
+  for (i = len;
+       (fullFileName[i] != '/') && (i > 0);
+       i--);
+         
+  fullDirName = (char *) malloc(i + 1);    
+  strncpy(fullDirName, fullFileName, i + 1);  
+  
+  if(strcmp(fullDirName, options.mp3loaddir) != 0)
+     strncpy(options.mp3loaddir, fullDirName, 254);
+
   return fullFileName;
 }
 
@@ -92,11 +100,16 @@ GtkWidget *createFileDialog(char *title, GtkSignalFunc addFunc,
   GtkWidget *hbox;  
   
   fileSel = GTK_FILE_SELECTION(gtk_file_selection_new(title));
+  gtk_window_set_wmclass(GTK_WINDOW(fileSel), "gmp3.file_selection", NULL);
+
+  /* set the last-used-dir [pel] */
+  gtk_file_selection_set_filename(fileSel, options.mp3loaddir);  
+
   fileSelWid = GTK_WIDGET(fileSel);
   gtk_file_selection_hide_fileop_buttons(fileSel);
   
-  gtk_widget_destroy(fileSel->ok_button);
-  gtk_widget_destroy(fileSel->cancel_button);
+  gtk_widget_hide(fileSel->ok_button);
+  gtk_widget_hide(fileSel->cancel_button);    
   
   gtk_clist_set_selection_mode(GTK_CLIST(fileSel->file_list),
                                GTK_SELECTION_MULTIPLE);
@@ -107,34 +120,41 @@ GtkWidget *createFileDialog(char *title, GtkSignalFunc addFunc,
   
   button = gtk_button_new_with_label("Add");
   gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-  fileSel->ok_button = button;
-  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);  
-  gtk_widget_show(button);    
+  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);    
+  gtk_widget_grab_default(button);
+  gtk_widget_show(button);
+
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     addFunc, fileSel);
   
   dirButton = gtk_button_new_with_label("Add Dir");
   gtk_box_pack_start(GTK_BOX(hbox), dirButton, TRUE, TRUE, 0);
   GTK_WIDGET_SET_FLAGS(dirButton, GTK_CAN_DEFAULT);
   gtk_widget_show(dirButton);
+
+  gtk_signal_connect(GTK_OBJECT(dirButton), "clicked",
+                     addDirFunc, fileSel);  
   
   button = gtk_button_new_with_label("Close");
   gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-  fileSel->cancel_button = button;
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
   gtk_widget_show(button);  
+
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(closeDialog), fileSelWid);
   
   gtk_signal_connect(GTK_OBJECT(fileSel), "destroy",
                      GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
   gtk_signal_connect(GTK_OBJECT(fileSel), "delete_event",
                      GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
-                     
-  gtk_signal_connect(GTK_OBJECT(fileSel->ok_button), "clicked",
-                     addFunc, fileSel);
-  gtk_signal_connect(GTK_OBJECT(dirButton), "clicked",
-                     addDirFunc, fileSel);
-  gtk_signal_connect(GTK_OBJECT(fileSel->cancel_button), "clicked",
-                     GTK_SIGNAL_FUNC(closeDialog), fileSelWid);
-                     
-  return fileSelWid;
+                       
+#ifdef __DEBUG__
+  g_print("File Selection Created: %s\n", title);
+#endif
+
+  gtk_widget_grab_focus(fileSel->selection_entry);
+ 
+  return fileSelWid;  
 }                                         
 
 /*****************
@@ -236,13 +256,15 @@ void loadAlbumns(char *fileName)
     
   fgets(tempBuffer, 255, inputFile);
   chop(tempBuffer);
-    
+  
   if (strcmp(tempBuffer, albumnlist_id_string) != 0)
     {
       fclose(inputFile);
       return;
     }
     
+  strncpy(options.playListName, fileName, 1023);
+    
   destroyPlaylist();
   initializePlaylist();
   
@@ -292,6 +314,7 @@ void loadAlbumns(char *fileName)
     }
     
   albumnList = albumnStack[1];
+  albumnChanged = FALSE;
 
   fclose(inputFile);
 }
@@ -326,7 +349,7 @@ void doSaveAlbumns(albumnNode *albumn, FILE *outputFile, int root)
       currentAlbumn = currentAlbumn->next;
     }
     
-  fprintf(outputFile, "\\\n");
+  fprintf(outputFile, "\\\n");  
 }
 
 /*
@@ -344,6 +367,7 @@ void saveAlbumns(char *fileName)
     }  
   
   fclose(outputFile);    
+  albumnChanged = FALSE;
 }
 
 /******************************
@@ -450,7 +474,7 @@ void addFileToPlayList(char *fileName)
   songNode *song;
   
   song = newSong(fileName, NULL, NULL);
-  g_list_append(activePlayList, song);
+  g_list_append(activePlayList, song);  
 }
 
 void clearPlayedFlags()
@@ -592,6 +616,8 @@ void createRootAlbumn(GtkWidget *widget, GtkEntry *entry)
   gtk_object_set_data(GTK_OBJECT(treeItem), tree_item_id_string, albumn);
   gtk_widget_show(treeItem);
   gtk_tree_append(GTK_TREE(tree), treeItem);
+  
+  albumnChanged = TRUE;
 }
 
 void createAlbumn(GtkWidget *widget, GtkEntry *entry)
@@ -630,6 +656,8 @@ void createAlbumn(GtkWidget *widget, GtkEntry *entry)
   gtk_object_set_data(GTK_OBJECT(treeItem), tree_item_id_string, albumn);
   gtk_widget_show(treeItem);
   gtk_tree_append(GTK_TREE(newTree), treeItem);
+  
+  albumnChanged = TRUE;
 }
 
 void createAlbumnDialog()
@@ -769,6 +797,8 @@ void deleteAlbumn()
   
   g_list_remove(parentAlbumn->albumns, albumn);  
   gtk_signal_emit_by_name(GTK_OBJECT(tree), "selection_changed");
+  
+  albumnChanged = TRUE;
 }
 
 /*
@@ -805,6 +835,8 @@ void changeAlbumnName(GtkWidget *widget, gint command)
       gtk_label_set(GTK_LABEL(GTK_BIN(item)->child), text);      
       entry = NULL;
     }      
+    
+  albumnChanged = TRUE;
 }
 
 void renameAlbumnDialog()
@@ -890,12 +922,12 @@ void createTreeMenu()
   gtk_signal_connect(GTK_OBJECT(menuItem), "activate",
                      GTK_SIGNAL_FUNC(addWholeAlbumn), NULL);
   gtk_widget_show(menuItem);
-                     
+
   menuItem = gtk_menu_item_new_with_label("Create Albumn...");
   gtk_menu_append(GTK_MENU(treeMenu), menuItem);
   gtk_signal_connect(GTK_OBJECT(menuItem), "activate",
                      GTK_SIGNAL_FUNC(createAlbumnDialog), tree);
-  gtk_widget_show(menuItem);    
+  gtk_widget_show(menuItem); 
 
   menuItem = gtk_menu_item_new_with_label("Create Sub-Albumn...");
   gtk_menu_append(GTK_MENU(treeMenu), menuItem);
@@ -956,51 +988,68 @@ void copyToPlayList()
 
 void addToSongList(GtkWidget *widget, GtkFileSelection *fileSel, albumnNode *albumn)
 {
-  GList *selections;
-  static albumnNode *currentAlbumn = NULL;  
+  GList *selections, *selection;
   songNode *song;
-  char *fileName;
+  char *fileName;  
+  char *url;
   gint row;
+  albumnNode *currentAlbumn;
   
-  if ((widget == NULL) && (fileSel == NULL))
+  selection = GTK_TREE_SELECTION(tree);
+  if (! selection)
     {
-      currentAlbumn = albumn;
       return;
+    }  
+  currentAlbumn = gtk_object_get_data(GTK_OBJECT(selection->data), tree_item_id_string);
+    
+  selections = GTK_CLIST(fileSel->file_list)->selection;    
+  fileName = gtk_entry_get_text(GTK_ENTRY(fileSel->selection_entry));
+ 
+  if (selections)
+    while (selections)
+      {
+        row = (gint)(selections->data);
+        gtk_clist_get_text(GTK_CLIST(fileSel->file_list), row, 0, &fileName);
+        song = newSong(completeFileName(fileSel, fileName), NULL, currentAlbumn);
+        g_list_append(currentAlbumn->songs, song);
+        selections = selections->next;
+      }
+  else if (strlen(fileName) > 0)
+    {
+      url = (char *)malloc(8);
+      strncpy(url, fileName, 7);
+      url[7] = '\0';
+      
+      if (strcasecmp(url, "http://") != 0)
+        song = newSong(completeFileName(fileSel, fileName), NULL, currentAlbumn);
+      else
+        song = newSong(fileName, NULL, currentAlbumn);        
+        
+      g_list_append(currentAlbumn->songs, song);
+      free(url);
     }
-  else if (currentAlbumn == NULL)
+  else
     return;
     
-  selections = GTK_CLIST(fileSel->file_list)->selection;
-  
-  while (selections)
-    {
-      row = (gint)(selections->data);
-      gtk_clist_get_text(GTK_CLIST(fileSel->file_list), row, 0, &fileName);
-      song = newSong(completeFileName(fileSel, fileName), NULL, currentAlbumn);
-      g_list_append(currentAlbumn->songs, song);
-      selections = selections->next;
-    }
-    
   treeSelectionChanged();
-  
-  currentAlbumn = NULL;
+  albumnChanged = TRUE;
 }
 
 void addDirToSongList(GtkWidget *widget, GtkFileSelection *fileSel, albumnNode *albumn)
 {
-  static albumnNode *currentAlbumn = NULL;  
+  albumnNode *currentAlbumn;
   songNode *song;
   char *fileName;
   gint row;
-  
-  if ((widget == NULL) && (fileSel == NULL))
+  GList *selection;
+
+  selection = GTK_TREE_SELECTION(tree);
+  if (! selection)
     {
-      currentAlbumn = albumn;
       return;
-    }
-  else if (currentAlbumn == NULL)
-    return;
-    
+    }  
+  currentAlbumn = gtk_object_get_data(GTK_OBJECT(selection->data), tree_item_id_string);
+  
   for (row = 0; row < GTK_CLIST(fileSel->file_list)->rows; row++)
     {
       gtk_clist_get_text(GTK_CLIST(fileSel->file_list), row, 0, &fileName);
@@ -1009,29 +1058,20 @@ void addDirToSongList(GtkWidget *widget, GtkFileSelection *fileSel, albumnNode *
     }
     
   treeSelectionChanged();
-  
-  currentAlbumn = NULL;
+  albumnChanged = TRUE;
 }
 
 void addToSongListDialog()
 {
   static GtkWidget *fileSel = NULL;
   GList *selection;
-  albumnNode *albumn;
       
   selection = GTK_TREE_SELECTION(tree);
-  if (! selection)
+  if (selection == NULL)
     return;
-  else if ((! fileSel) || (! GTK_WIDGET_VISIBLE(fileSel)))
-    {
-      albumn = gtk_object_get_data(GTK_OBJECT(selection->data),
-                                   tree_item_id_string);    
-      addToSongList(NULL, NULL, albumn);
-      addDirToSongList(NULL, NULL, albumn);
-    }
   
   if (! fileSel)
-    {      
+    {
       fileSel = createFileDialog("Add to Albumn", 
                                  GTK_SIGNAL_FUNC(addToSongList),
                                  GTK_SIGNAL_FUNC(addDirToSongList));
@@ -1092,6 +1132,7 @@ void deleteFromSongList()
 
   g_list_free(songTitles);    
   gtk_clist_thaw(GTK_CLIST(songlist));
+  albumnChanged = TRUE;
 }
 
 /*
@@ -1128,6 +1169,8 @@ void changeSongProperties(GtkWidget *widget, gint row)
       
       entry = NULL;
     }
+    
+  albumnChanged = TRUE;
 }
 
 void songPropertiesDialog()
@@ -1311,23 +1354,44 @@ void addToPlayList(GtkWidget *widget, GtkFileSelection *fileSel, albumnNode *alb
 {
   GList *selections;
   songNode *song;
+  char *url;
   char *fileName;
-  gint row;
+  gint row = 0;
   
   selections = GTK_CLIST(fileSel->file_list)->selection;
+  fileName = gtk_entry_get_text(GTK_ENTRY(fileSel->selection_entry));
   
-  while (selections)
-    {
-      row = (gint)(selections->data);
-      gtk_clist_get_text(GTK_CLIST(fileSel->file_list), row, 0, &fileName);
-      song = newSong(completeFileName(fileSel, fileName), NULL, NULL);
+  if (selections)
+    while (selections)
+      {
+        row = (gint)(selections->data);
+        gtk_clist_get_text(GTK_CLIST(fileSel->file_list), row, 0, &fileName);
+        song = newSong(completeFileName(fileSel, fileName), NULL, NULL);
       
+        gtk_clist_append(GTK_CLIST(playlist), &(song->title));
+        gtk_clist_set_row_data(GTK_CLIST(playlist), row, song);
+            
+        g_list_append(activePlayList, song);
+        selections = selections->next;
+      }
+  else if (strlen(fileName) > 0)
+    {
+      url = (char *)malloc(8);
+      strncpy(url, fileName, 7);
+      url[7] = '\0';
+      
+      if (strcasecmp(url, "http://") != 0)
+        song = newSong(completeFileName(fileSel, fileName), NULL, NULL);
+      else
+        song = newSong(fileName, NULL, NULL);
+
       gtk_clist_append(GTK_CLIST(playlist), &(song->title));
       gtk_clist_set_row_data(GTK_CLIST(playlist), row, song);
-            
+        
       g_list_append(activePlayList, song);
-      selections = selections->next;
     }
+  else
+    return;
 }
 
 void addDirToPlayList(GtkWidget *widget, GtkFileSelection *fileSel, albumnNode *albumn)
@@ -1354,7 +1418,7 @@ void addToPlayListDialog()
       
   if (! fileSel)
     {      
-      fileSel = createFileDialog("Add to Albumn", 
+      fileSel = createFileDialog("Add to PlayList", 
                                  GTK_SIGNAL_FUNC(addToPlayList),
                                  GTK_SIGNAL_FUNC(addDirToPlayList));
       gtk_widget_show(fileSel);
@@ -1460,6 +1524,8 @@ void loadAlbumnCallback(GtkWidget *widget, GtkFileSelection *fileSel)
 void loadAlbumnDialog()
 {
   static GtkFileSelection *fileSel = NULL;
+  char *albumnDir;
+  char *homeDir = getenv("HOME");
   
   if (! fileSel)
     {
@@ -1480,6 +1546,14 @@ void loadAlbumnDialog()
       gtk_signal_connect(GTK_OBJECT(fileSel->cancel_button), "clicked",
                          GTK_SIGNAL_FUNC(closeDialog), fileSel);
                          
+      /* set cwd to the albumn directory [pel] */
+      
+      albumnDir = (char *)malloc(strlen(homeDir) + strlen(ALBUMNSUBDIR) + 3);
+      strcpy(albumnDir, ALBUMNSUBDIR);
+      strcat(albumnDir, "/");
+      strcat(albumnDir, ALBUMNSUBDIR);
+      
+      gtk_file_selection_set_filename(fileSel, albumnDir);
       gtk_widget_show(GTK_WIDGET(fileSel));
     }    
   else
@@ -1497,6 +1571,8 @@ void saveAlbumnCallback(GtkWidget *widget, GtkFileSelection *fileSel)
 void saveAlbumnDialog()
 {
   static GtkFileSelection *fileSel = NULL;
+  char *albumnDir;
+  char *homeDir = getenv("HOME");
   
   if (! fileSel)
     {
@@ -1517,6 +1593,13 @@ void saveAlbumnDialog()
       gtk_signal_connect(GTK_OBJECT(fileSel->cancel_button), "clicked",
                          GTK_SIGNAL_FUNC(closeDialog), fileSel);
                          
+      /* set cwd to the albumn directory [pel] */
+      
+      albumnDir = (char *)malloc(strlen(homeDir) + strlen(ALBUMNSUBDIR) + 3);
+      strcpy(albumnDir, homeDir);
+      strcat(albumnDir, ALBUMNSUBDIR);      
+
+      gtk_file_selection_set_filename(fileSel, albumnDir);
       gtk_widget_show(GTK_WIDGET(fileSel));
     }    
   else
@@ -1604,201 +1687,244 @@ void savePlayListDialog()
 }
  
 
-/****************************************
-  Routines for creating the list dialog. 
- ****************************************/
+/***************************************************************
+  Routines for creating the playlist and albumn editor dialogs. 
+ ***************************************************************/
 
-void createListDialog()
+void createPlaylistEditorDialog()
 {
-  static GtkWidget *editorWindow;
-  static GtkWidget *playlistWindow;  
+  GtkWidget *hbox;
+  GtkWidget *vbox;
+  GtkWidget *button;
+  GtkWidget *eventBox;
+  GtkTooltips *tooltips;
+  songNode *song;
+  GList *currentSong;
+  gint row;
+
+  if (options.toolTips)
+    tooltips = gtk_tooltips_new();
+  else
+    tooltips = NULL;
   
+  playlistWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title(GTK_WINDOW(playlistWindow), "Active Playlist");
+  gtk_window_set_wmclass(GTK_WINDOW(playlistWindow), "gmp3.active_playlist", NULL);
+  gtk_signal_connect(GTK_OBJECT(playlistWindow), "delete_event",
+                     GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
+  
+  vbox = gtk_vbox_new(FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(playlistWindow), vbox);
+  gtk_widget_show(vbox);
+  
+  /* Setup CList */
+  
+  eventBox = gtk_event_box_new();
+  gtk_box_pack_start(GTK_BOX(vbox), eventBox, TRUE, TRUE, 0);
+  gtk_widget_show(eventBox);
+  
+  playlist = gtk_clist_new(1);
+  gtk_clist_set_selection_mode(GTK_CLIST(playlist), GTK_SELECTION_MULTIPLE);
+  gtk_widget_set_usize(playlist, 200, 150);
+  gtk_widget_show(playlist);
+  gtk_container_add(GTK_CONTAINER(eventBox), playlist);
+  
+  if (tooltips)
+    gtk_tooltips_set_tip(tooltips, eventBox, "Right-click for menu", NULL);
+  
+  currentSong = g_list_first(activePlayList)->next;
+  while (currentSong)
+    {
+      song = (songNode *)(currentSong->data);
+      row = gtk_clist_append(GTK_CLIST(playlist), &(song->title));
+      gtk_clist_set_row_data(GTK_CLIST(playlist), row, song);
+      
+      currentSong = currentSong->next;
+    }
+  
+  createPlayListMenu();
+  gtk_signal_connect(GTK_OBJECT(playlist), "button_press_event",
+                     GTK_SIGNAL_FUNC(popupMenu), &playlistMenu);
+  
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+  gtk_widget_show(hbox);
+  
+  button = gtk_button_new_with_label("Clear");
+  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+  gtk_widget_show(button);
+  
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(clearPlayList), NULL);
+  
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+  gtk_widget_show(hbox);
+  
+  button = gtk_button_new_with_label("Load");
+  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+  gtk_widget_show(button);
+  
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(loadPlayListDialog), NULL);
+  
+  button = gtk_button_new_with_label("Save");
+  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+  gtk_widget_show(button);
+  
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(savePlayListDialog), NULL);
+  
+  button = gtk_button_new_with_label("Close");
+  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+  gtk_widget_show(button);
+  
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(closeDialog), playlistWindow);
+  
+  insertCListHandlers(playlist);      
+}
+
+void createAlbumnEditorDialog()
+{
   GtkWidget *scrolledWindow;
   GtkWidget *hbox;
   GtkWidget *vbox;
   GtkWidget *button;
   GtkWidget *eventBox;
-  songNode *song;
-  GList *currentSong;
-  gint row;
-    
-  if (! playlistWindow)
-    {
-      playlistWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-      gtk_window_set_title(GTK_WINDOW(playlistWindow), "Active Playlist");
-      gtk_window_set_wmclass(GTK_WINDOW(playlistWindow), "gmp3.active_playlist", NULL);
-      gtk_signal_connect(GTK_OBJECT(playlistWindow), "delete_event",
-                         GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
-      
-      
-      vbox = gtk_vbox_new(FALSE, 0);
-      gtk_container_add(GTK_CONTAINER(playlistWindow), vbox);
-      gtk_widget_show(vbox);
-      
-      /* Setup CList */
+  GtkTooltips *tooltips;
 
-      playlist = gtk_clist_new(1);
-      gtk_clist_set_selection_mode(GTK_CLIST(playlist), GTK_SELECTION_MULTIPLE);
-      gtk_widget_set_usize(playlist, 200, 150);
-      gtk_widget_show(playlist);
-      gtk_box_pack_start(GTK_BOX(vbox), playlist, TRUE, TRUE, 0);
-      
-      currentSong = g_list_first(activePlayList)->next;
-      while (currentSong)
-        {
-          song = (songNode *)(currentSong->data);
-          row = gtk_clist_append(GTK_CLIST(playlist), &(song->title));
-          gtk_clist_set_row_data(GTK_CLIST(playlist), row, song);
-          
-          currentSong = currentSong->next;
-        }
-      
-      createPlayListMenu();
-      gtk_signal_connect(GTK_OBJECT(playlist), "button_press_event",
-                         GTK_SIGNAL_FUNC(popupMenu), &playlistMenu);
+  if (! playlistWindow) createPlaylistEditorDialog();
 
-      hbox = gtk_hbox_new(FALSE, 0);
-      gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-      gtk_widget_show(hbox);
-                         
-      button = gtk_button_new_with_label("Clear");
-      gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-      gtk_widget_show(button);
-      
-      gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                         GTK_SIGNAL_FUNC(clearPlayList), NULL);
-                        
-      hbox = gtk_hbox_new(FALSE, 0);
-      gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-      gtk_widget_show(hbox);
-                         
-      button = gtk_button_new_with_label("Load");
-      gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-      gtk_widget_show(button);
-      
-      gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                         GTK_SIGNAL_FUNC(loadPlayListDialog), NULL);
-
-      button = gtk_button_new_with_label("Save");
-      gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-      gtk_widget_show(button);
-
-      gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                         GTK_SIGNAL_FUNC(savePlayListDialog), NULL);
-
-      button = gtk_button_new_with_label("Close");
-      gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-      gtk_widget_show(button);
-
-      gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                         GTK_SIGNAL_FUNC(closeDialog), playlistWindow);
-                                                       
-      insertCListHandlers(playlist);      
-    }
-
-  if (! editorWindow)
-    {          
-      editorWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-      gtk_window_set_title(GTK_WINDOW(editorWindow), "Albumn Editor");
-      gtk_window_set_wmclass(GTK_WINDOW(editorWindow), "gmp3.albumn_editor", NULL);
-      gtk_signal_connect(GTK_OBJECT(editorWindow), "delete_event",
-                         GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
-
-      vbox = gtk_vbox_new(FALSE, 0);
-      gtk_container_add(GTK_CONTAINER(editorWindow), vbox);
-      gtk_widget_show(vbox);
-
-      hbox = gtk_hbox_new(FALSE, 5);
-      gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-      gtk_widget_show(hbox);
-
-      /* Tree Setup Code */
-
-      createTreeMenu();
-      
-      eventBox = gtk_event_box_new();
-      gtk_box_pack_start(GTK_BOX(hbox), eventBox, FALSE, FALSE, 0);
-      gtk_widget_show(eventBox);
-      
-      scrolledWindow = gtk_scrolled_window_new(NULL, NULL);
-      gtk_widget_show(scrolledWindow);
-      gtk_widget_set_usize(scrolledWindow, 150, 150);
-      gtk_container_add(GTK_CONTAINER(eventBox), scrolledWindow);
-
-      tree = gtk_tree_new();
-      gtk_object_set_data(GTK_OBJECT(tree), tree_item_id_string, &albumnList);
-      constructTree(tree, albumnList, 0);
-      gtk_widget_show(tree);
-      gtk_container_add(GTK_CONTAINER(scrolledWindow), tree);
-        
-      gtk_signal_connect(GTK_OBJECT(eventBox), "button_press_event",
-                         GTK_SIGNAL_FUNC(popupMenu), &treeMenu);
-            
-      insertTreeHandlers(tree);      
-
-      /* End Tree Setup */
-
-      /* Setup CList */
-
-      songlist = gtk_clist_new(1);
-      gtk_clist_set_selection_mode(GTK_CLIST(songlist), GTK_SELECTION_MULTIPLE);
-      gtk_widget_set_usize(songlist, 200, 150);
-      gtk_widget_show(songlist);
-      gtk_box_pack_start(GTK_BOX(hbox), songlist, TRUE, TRUE, 0);
-      
-      createSongListMenu();
-      gtk_signal_connect(GTK_OBJECT(songlist), "button_press_event",
-                         GTK_SIGNAL_FUNC(popupMenu), &songMenu);
-
-      /* End CList Setup */
-
-      /* Setup Control Buttons */
-
-      hbox = gtk_hbox_new(FALSE, 0);
-      gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-      gtk_widget_show(hbox);
-                         
-      button = gtk_button_new_with_label("Active Playlist");
-      gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-      gtk_widget_show(button);
-      
-      gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                         GTK_SIGNAL_FUNC(openDialog), playlistWindow);
-
-      hbox = gtk_hbox_new(FALSE, 0);
-      gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-      gtk_widget_show(hbox);
-                         
-      button = gtk_button_new_with_label("Load");
-      gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-      gtk_widget_show(button);
-      
-      gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                         GTK_SIGNAL_FUNC(loadAlbumnDialog), NULL);
-
-      button = gtk_button_new_with_label("Save");
-      gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-      gtk_widget_show(button);
-
-      gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                         GTK_SIGNAL_FUNC(saveAlbumnDialog), NULL);
-
-      button = gtk_button_new_with_label("Close");
-      gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-      gtk_widget_show(button);
-
-      gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                         GTK_SIGNAL_FUNC(closeDialog), editorWindow);
-                         
-      /* Set up tree signal */
-
-      gtk_signal_connect(GTK_OBJECT(tree), "selection_changed",
-                         GTK_SIGNAL_FUNC(treeSelectionChanged), NULL);
-
-      /* Done tree signal */
-
-      gtk_widget_show(editorWindow);
-    }
+  if (options.toolTips)
+    tooltips = gtk_tooltips_new();
   else
-    openDialog(NULL, editorWindow);
+    tooltips = NULL;
+  
+  editorWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title(GTK_WINDOW(editorWindow), "Albumn Editor");
+  gtk_window_set_wmclass(GTK_WINDOW(editorWindow), "gmp3.albumn_editor", NULL);
+  gtk_signal_connect(GTK_OBJECT(editorWindow), "delete_event",
+                     GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
+  
+  vbox = gtk_vbox_new(FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(editorWindow), vbox);
+  gtk_widget_show(vbox);
+
+  hbox = gtk_hbox_new(FALSE, 5);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+  gtk_widget_show(hbox);
+      
+  /* Tree Setup Code */
+
+  createTreeMenu();
+  
+  eventBox = gtk_event_box_new();
+  gtk_box_pack_start(GTK_BOX(hbox), eventBox, FALSE, FALSE, 0);      
+  gtk_widget_show(eventBox);
+  
+  scrolledWindow = gtk_scrolled_window_new(NULL, NULL);
+  gtk_widget_show(scrolledWindow);
+  gtk_widget_set_usize(scrolledWindow, 150, 150);
+  gtk_container_add(GTK_CONTAINER(eventBox), scrolledWindow);
+  
+  tree = gtk_tree_new();
+  gtk_object_set_data(GTK_OBJECT(tree), tree_item_id_string, &albumnList); 
+  constructTree(tree, albumnList, 0);
+  gtk_widget_show(tree);
+  gtk_container_add(GTK_CONTAINER(scrolledWindow), tree); 
+  gtk_signal_connect(GTK_OBJECT(eventBox), "button_press_event",
+                     GTK_SIGNAL_FUNC(popupMenu), &treeMenu);                               
+  if (tooltips)
+    gtk_tooltips_set_tip(tooltips, eventBox, "Right-click for menu", NULL);
+  
+  insertTreeHandlers(tree);
+  
+  /* End Tree Setup */
+  
+  /* Setup CList */
+  
+  eventBox = gtk_event_box_new();
+  gtk_box_pack_start(GTK_BOX(hbox), eventBox, TRUE, TRUE, 0);
+  gtk_widget_show(eventBox);      
+  
+  songlist = gtk_clist_new(1);
+  gtk_clist_set_selection_mode(GTK_CLIST(songlist), GTK_SELECTION_MULTIPLE);
+  gtk_widget_set_usize(songlist, 200, 150);
+  gtk_widget_show(songlist);
+  gtk_container_add(GTK_CONTAINER(eventBox), songlist);
+  
+  createSongListMenu();
+  gtk_signal_connect(GTK_OBJECT(songlist), "button_press_event",
+                     GTK_SIGNAL_FUNC(popupMenu), &songMenu);
+  
+  if (tooltips)
+    gtk_tooltips_set_tip(tooltips, eventBox, "Right-click for menu", NULL);
+  
+  /* End CList Setup */
+  
+  /* Setup Control Buttons */
+  
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+  gtk_widget_show(hbox);
+  
+  button = gtk_button_new_with_label("Active Playlist");
+  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+  gtk_widget_show(button);
+  
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(openPlaylistEditorDialog), NULL);
+  
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+  gtk_widget_show(hbox);
+  
+  button = gtk_button_new_with_label("Load");
+  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+  gtk_widget_show(button);
+  
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(loadAlbumnDialog), NULL);
+  
+  button = gtk_button_new_with_label("Save");
+  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+  gtk_widget_show(button);
+  
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(saveAlbumnDialog), NULL);
+  
+  button = gtk_button_new_with_label("Close");
+  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+  gtk_widget_show(button);
+  
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(closeDialog), editorWindow);
+  
+  /* Set up tree signal */
+  
+  gtk_signal_connect(GTK_OBJECT(tree), "selection_changed",
+                     GTK_SIGNAL_FUNC(treeSelectionChanged), NULL);
+  
+  /* Done tree signal */
+}
+
+/*****************************************************
+  Public interface functions for dialog display code.
+ *****************************************************/
+
+void openAlbumnEditorDialog()
+{
+  if (! editorWindow)
+    createAlbumnEditorDialog();
+  
+  openDialog(NULL, editorWindow);
+}
+
+void openPlaylistEditorDialog()
+{
+  if (! playlistWindow)
+    createPlaylistEditorDialog();
+  
+  openDialog(NULL, playlistWindow);
 }
