@@ -27,6 +27,7 @@
 extern albumnNode *albumnList;
 extern GList *activePlayList;
 extern GList *currentSong;
+extern int albumnChanged;
 
 void initializeAlbumnList(albumnNode **albumn);
 void destroyAlbumnList(albumnNode *albumn);
@@ -34,7 +35,8 @@ void destroyAlbumnList(albumnNode *albumn);
 void initializePlaylist();
 void destroyPlaylist();
 
-void createListDialog();
+void openPlaylistEditorDialog();
+void openAlbumnEditorDialog();
 void clearPlayedFlags();
 
 void loadAlbumns(char *fileName);
@@ -44,4 +46,4 @@ void addFileToPlayList(char *fileName);
 songNode *newSong(char *fileName, char *title, albumnNode *parentAlbumn);
 albumnNode *newAlbumn(char *title, albumnNode *parent);
 
-#endif
\ No newline at end of file
+#endif
