@@ -1,6 +1,4 @@
-
-/* GMP3 - A front end for mpg123
- * Copyright (C) 1998 Brett Kosinski
+/* GMP3 - A front end for mpg123 Copyright (C) 1998 Brett Kosinski
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +30,50 @@ void toggleVisible(GtkWidget *wid, GtkWidget *widget)
     gtk_widget_hide(widget);
 }
 
+void mp3AppEntryCallback(GtkWidget *widget, gpointer *data)
+{
+  char *entryString;
+
+  entryString = gtk_entry_get_text(GTK_ENTRY(widget));
+  if (strlen(entryString) == 0)     
+    options.mp3player[0] = '\0';
+  else
+    strncpy(options.mp3player, entryString, 255);
+}
+
+void soundDeviceEntryCallback(GtkWidget *widget, gpointer *data)
+{
+  char *entryString;
+
+  entryString = gtk_entry_get_text(GTK_ENTRY(widget));
+  if (strlen(entryString) == 0)     
+    options.soundDevice[0] = '\0';
+  else
+    strncpy(options.soundDevice, entryString, 255);
+}
+
+void httpProxyCallback(GtkWidget *widget, gpointer *data)
+{
+  char *entryString;
+
+  entryString = gtk_entry_get_text(GTK_ENTRY(widget));
+  if (strlen(entryString) == 0)     
+    options.httpProxy[0] = '\0';
+  else
+    strncpy(options.httpProxy, entryString, 1023);
+}
+
+void httpAuthStringCallback(GtkWidget *widget, gpointer *data)
+{
+  char *entryString;
+
+  entryString = gtk_entry_get_text(GTK_ENTRY(widget));
+  if (strlen(entryString) == 0)     
+    options.httpAuthString[0] = '\0';
+  else
+    strncpy(options.httpAuthString, entryString, 1023);
+}
+
 void bufferEntryCallback(GtkWidget *widget, gpointer *data)
 {
   char *entryString;
@@ -87,68 +129,6 @@ void autoparseCallback(GtkWidget *widget, gpointer *data)
   options.autoparse = ! options.autoparse;
 }
 
-void autoloadEntryCallback(GtkWidget *widget, gpointer *data)
-{
-  char *entryString;
-
-  entryString = gtk_entry_get_text(GTK_ENTRY(widget));
-  if (strlen(entryString) == 0)     
-    options.playListName[0] = '\0';
-  else
-    strncpy(options.playListName, entryString, 255);
-}
-
-void noAutoloadCallback(GtkWidget *widget, GtkWidget *entry)
-{
-  gtk_entry_set_text(GTK_ENTRY(entry), "");    
-  options.playListName[0] = '\0';
-}
-
-void autoloadFileSelected(GtkWidget *widget, GtkEntry *entry)
-{
-  char *entryString;
-
-  entryString = gtk_file_selection_get_filename(autoloadSel);
-  
-  if (strlen(entryString) == 0)     
-    options.playListName[0] = '\0';
-  else
-    strncpy(options.playListName, entryString, 255);  
-    
-  gtk_entry_set_text(entry, options.playListName);
-}
-
-void createAutoloadFileSel(GtkWidget *widget, gpointer *entry)
-{
-  if (!autoloadSel)
-    {
-      autoloadSel = GTK_FILE_SELECTION(gtk_file_selection_new("Select Playlist"));
-      
-      gtk_signal_connect(GTK_OBJECT(autoloadSel->ok_button), "clicked",
-                         GTK_SIGNAL_FUNC(autoloadFileSelected), 
-                         entry);
-      
-      gtk_signal_connect_object(GTK_OBJECT(autoloadSel->ok_button), "clicked",
-                                GTK_SIGNAL_FUNC(gtk_widget_hide),
-                                GTK_OBJECT(autoloadSel));                         
-      gtk_signal_connect_object(GTK_OBJECT(autoloadSel->cancel_button), "clicked",
-                                GTK_SIGNAL_FUNC(gtk_widget_hide),
-                                GTK_OBJECT(autoloadSel));
-      gtk_signal_connect_object(GTK_OBJECT(autoloadSel), "delete_event",
-                                GTK_SIGNAL_FUNC(gtk_widget_hide),
-                                GTK_OBJECT(autoloadSel));                                
-                         
-      gtk_widget_show(GTK_WIDGET(autoloadSel));
-    }
-  else
-    {
-      if (! GTK_WIDGET_VISIBLE(autoloadSel))
-        gtk_widget_show(GTK_WIDGET(autoloadSel));
-      else
-        return;  
-    }    
-}
-
 void mixerAppEntryCallback(GtkWidget *widget, gpointer *data)
 {
   char *entryString;
@@ -276,7 +256,93 @@ void createOptionsDialog()
   gtk_widget_show(label);
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);
       
-/* Set up the buffer size widgets. */
+/* MP3 Player Stuff */
+
+  hbox = gtk_hbox_new(FALSE, 5);
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+
+  label = gtk_label_new("MP3 Player");
+  gtk_widget_show(label);
+  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);  
+
+  entry = gtk_entry_new();
+  gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+  gtk_widget_show(entry);
+  
+  gtk_entry_set_text(GTK_ENTRY(entry), options.mp3player);
+  
+  gtk_signal_connect(GTK_OBJECT(entry), "changed",
+                     GTK_SIGNAL_FUNC(mp3AppEntryCallback), NULL);
+      
+/* Sound Device Stuff */
+
+  hbox = gtk_hbox_new(FALSE, 5);
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+
+  label = gtk_label_new("Sound Device");
+  gtk_widget_show(label);
+  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);  
+
+  entry = gtk_entry_new();
+  gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+  gtk_widget_show(entry);
+  
+  gtk_entry_set_text(GTK_ENTRY(entry), options.soundDevice);
+  
+  gtk_signal_connect(GTK_OBJECT(entry), "changed",
+                     GTK_SIGNAL_FUNC(soundDeviceEntryCallback), NULL);
+
+  separator = gtk_hseparator_new();
+  gtk_widget_show(separator);
+  gtk_box_pack_start(GTK_BOX(vbox), separator, TRUE, TRUE, 0);
+
+/* HTTP Options... */
+
+/* HTTP Proxy Server... */
+
+  hbox = gtk_hbox_new(FALSE, 5);
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+
+  label = gtk_label_new("HTTP Proxy");
+  gtk_widget_show(label);
+  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);  
+
+  entry = gtk_entry_new();
+  gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+  gtk_widget_show(entry);
+  
+  gtk_entry_set_text(GTK_ENTRY(entry), options.httpProxy);
+  
+  gtk_signal_connect(GTK_OBJECT(entry), "changed",
+                     GTK_SIGNAL_FUNC(httpProxyCallback), NULL);
+
+/* HTTP Auth String */
+
+  hbox = gtk_hbox_new(FALSE, 5);
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+
+  label = gtk_label_new("HTTP Auth");
+  gtk_widget_show(label);
+  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);  
+
+  entry = gtk_entry_new();
+  gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+  gtk_widget_show(entry);
+  
+  gtk_entry_set_text(GTK_ENTRY(entry), options.httpAuthString);
+  
+  gtk_signal_connect(GTK_OBJECT(entry), "changed",
+                     GTK_SIGNAL_FUNC(httpAuthStringCallback), NULL);
+
+  separator = gtk_hseparator_new();
+  gtk_widget_show(separator);
+  gtk_box_pack_start(GTK_BOX(vbox), separator, TRUE, TRUE, 0);
+
+/* Set up the buffer size widgets. */  
 
   hbox = gtk_hbox_new(FALSE, 5);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
@@ -305,10 +371,6 @@ void createOptionsDialog()
     
 /* Now the downsampling stuff */  
 
-  separator = gtk_hseparator_new();
-  gtk_widget_show(separator);
-  gtk_box_pack_start(GTK_BOX(vbox), separator, TRUE, TRUE, 0);
-
   label = gtk_label_new("Down Sampling");
   gtk_widget_show(label);
   gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);  
@@ -464,48 +526,6 @@ void createOptionsDialog()
   gtk_signal_connect(GTK_OBJECT(entry), "changed",
                      GTK_SIGNAL_FUNC(themeEntryCallback), NULL);    
   
-/* Playlist Autoload Gizmos */
-  
-  hbox = gtk_hbox_new(FALSE, 5);
-  gtk_widget_show(hbox);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-  button = gtk_check_button_new_with_label("Autoload");
-  gtk_widget_show(button);
-  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-  
-  entry = gtk_entry_new();
-  gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
- 
-  fileButton = gtk_button_new_with_label("...");
-  gtk_box_pack_start(GTK_BOX(hbox), fileButton, TRUE, TRUE, 0);
-  
-  if (options.playListName[0] != '\0')
-    {
-      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), TRUE);
-
-      strcpy(strBuffer, options.playListName);
-      gtk_entry_set_text(GTK_ENTRY(entry), strBuffer);   
-      
-      gtk_widget_show(entry);
-      gtk_widget_show(fileButton);
-    }          
-      
-  gtk_signal_connect(GTK_OBJECT(entry), "changed",
-                     GTK_SIGNAL_FUNC(autoloadEntryCallback), NULL);
-                       
-  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                     GTK_SIGNAL_FUNC(noAutoloadCallback), entry);                     
-  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                     GTK_SIGNAL_FUNC(toggleVisible), entry);                     
-  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                     GTK_SIGNAL_FUNC(toggleVisible), fileButton);
-                     
-  gtk_signal_connect(GTK_OBJECT(fileButton), "clicked",
-                     GTK_SIGNAL_FUNC(createAutoloadFileSel), entry);
-
-/* External Mixer App Gizmos */
-
   hbox = gtk_hbox_new(FALSE, 5);
   gtk_widget_show(hbox);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
@@ -543,7 +563,7 @@ void createOptionsDialog()
                      
   gtk_signal_connect(GTK_OBJECT(fileButton), "clicked",
                      GTK_SIGNAL_FUNC(createMixerAppFileSel), entry);
-
+                       
 /* Border checkbox */
 
   button = gtk_check_button_new_with_label("Window is managed");  
@@ -598,12 +618,6 @@ void createOptionsDialog()
   gtk_box_pack_start(GTK_BOX(mainvbox), hbox, TRUE, TRUE, 0);
   gtk_widget_show(hbox);    
   
-  button = gtk_button_new_with_label("Save");
-  gtk_widget_show(button);
-  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                     GTK_SIGNAL_FUNC(saveRCFile), NULL);
-  
   button = gtk_button_new_with_label("Close");
   gtk_widget_show(button);
   gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
@@ -616,4 +630,4 @@ void createOptionsDialog()
                             GTK_OBJECT(optionsDialog));                            
                                           
   gtk_widget_show(optionsDialog);
-}
\ No newline at end of file
+}
