@@ -18,9 +18,11 @@
  
 #ifndef __SHAPE__
 #define __SHAPE__
- 
-gint shape_pressed(GtkWidget *widget, GdkEventButton *event);
-gint shape_released(GtkWidget *widget);
-gint shape_motion(GtkWidget *widget, GdkEventMotion *event);
 
-#endif
\ No newline at end of file
+#include "themerc.h"
+ 
+gint shape_pressed(GtkWidget *widget, GdkEventButton *event, ThemeWindow *window);
+gint shape_released(GtkWidget *widget);
+gint shape_motion(GtkWidget *widget, GdkEventMotion *event, ThemeWindow *window);
+
+#endif
