@@ -294,6 +294,15 @@ void initializeWindow(ThemeWindow *window)
     window->Tooltips = gtk_tooltips_new();
 }
 
+void attachShapeData(GtkWidget *widget, ThemeWindow *window)
+{
+  gtk_object_set_user_data(GTK_OBJECT(widget), window->clickPos); 
+  gtk_signal_connect(GTK_OBJECT(widget), "button_release_event",
+                     GTK_SIGNAL_FUNC(shape_released), window);
+  gtk_signal_connect(GTK_OBJECT(widget), "motion_notify_event",
+                     GTK_SIGNAL_FUNC(shape_motion), window);               
+}
+
 gint parseAction(char *actionStr)
 {
   if (strcasecmp(actionStr, "cmd_no_command") == 0)
@@ -302,6 +311,8 @@ gint parseAction(char *actionStr)
     return CMD_BACK;
   if (strcasecmp(actionStr, "cmd_foward") == 0)
     return CMD_FORWARD;
+  if (strcasecmp(actionStr, "cmd_forward") == 0)
+    return CMD_FORWARD;        
   if (strcasecmp(actionStr, "cmd_play") == 0)
     return CMD_PLAY;
   if (strcasecmp(actionStr, "cmd_stop") == 0)
@@ -314,8 +325,10 @@ gint parseAction(char *actionStr)
     return CMD_TOGGLE_SHUFFLE;
   if (strcasecmp(actionStr, "cmd_toggle_repeat") == 0)
     return CMD_TOGGLE_REPEAT;
+  if (strcasecmp(actionStr, "cmd_open_albumn_editor") == 0)
+    return CMD_OPEN_ALBUMN_EDITOR;
   if (strcasecmp(actionStr, "cmd_open_playlist_editor") == 0)
-    return CMD_OPEN_PLAYLIST_EDITOR;
+    return CMD_OPEN_PLAYLIST_EDITOR;  
   if (strcasecmp(actionStr, "cmd_open_options") == 0)
     return CMD_OPEN_OPTIONS;
   if (strcasecmp(actionStr, "cmd_open_about_box") == 0)
@@ -326,6 +339,8 @@ gint parseAction(char *actionStr)
     return CMD_POPUP_MENU;
   if (strcasecmp(actionStr, "cmd_quit") == 0)
     return CMD_QUIT;
+  if (strcasecmp(actionStr, "cmd_drag_capture") == 0)
+    return CMD_DRAG_CAPTURE;
 
   rcFileError("invalid command string");
   return -1;
@@ -412,12 +427,12 @@ void parseLabel(ThemeLabel *label)
         continue;
       else if (strcasecmp(tokens.words[0], "font") == 0)
         {
-          strcpy(font, tokens.words[1]);
+          strncpy(font, tokens.words[1], 1023);
           gfont = gdk_font_load(font);
           if (gfont != NULL)
             style->font = gfont;
           else
-            rcFileError("unable to load font");     
+            rcFileError("unable to load font");    
         }  
       else if (strcasecmp(tokens.words[0], "color") == 0)
         {
@@ -437,8 +452,8 @@ void parseLabel(ThemeLabel *label)
                  tokens.words[3]);
       else if (strcasecmp(tokens.words[0], "action") == 0)
         label->widget->command = parseAction(tokens.words[1]);      
-      else
-        rcFileError("invalid identifier");
+      else 
+        rcFileError("invalid identifier"); 
     }
 
   label->eventBox = gtk_event_box_new();
@@ -499,6 +514,8 @@ void parseMenu(ThemeWindow *window)
     }
 }
 
+static gint emptyFunc(GtkWidget *widget) { return TRUE; }
+
 void parseVolumeSlider(ThemeWindow *window)
 {
   GtkObject *adjustment;
@@ -546,6 +563,9 @@ void parseVolumeSlider(ThemeWindow *window)
   gtk_scale_set_draw_value(GTK_SCALE(window->volume_slider.widget), FALSE);
   gtk_widget_set_usize(window->volume_slider.widget, width, height);
   gtk_widget_show(window->volume_slider.widget);
+  
+  gtk_signal_connect_after(GTK_OBJECT(window->volume_slider.widget), 
+                           "button_press_event", GTK_SIGNAL_FUNC(emptyFunc), NULL);
 }
 
 void parseButton(ThemeWindow *window)
@@ -624,31 +644,11 @@ void generateLabel(ThemeWindow *window, ThemeLabel *label)
     gtk_widget_set_usize(label->eventBox, label->width, label->height);
 
   gtk_widget_show(label->eventBox);
+  attachShapeData(label->eventBox, window);
 
-/* This reeks of hack... :) */
-  
-  if (label->widget->command == CMD_NO_COMMAND)
-    {
-      gtk_object_set_user_data(GTK_OBJECT(label->eventBox), window->clickPos);
-
-      gtk_signal_connect(GTK_OBJECT(label->eventBox), "button_press_event",
-                         GTK_SIGNAL_FUNC(shape_pressed), window);
-      gtk_signal_connect(GTK_OBJECT(label->eventBox), "button_release_event",
-                         GTK_SIGNAL_FUNC(shape_released), window);
-      gtk_signal_connect(GTK_OBJECT(label->eventBox), "motion_notify_event",
-                         GTK_SIGNAL_FUNC(shape_motion), window);             
-    }                         
-  else  
-    gtk_signal_connect(GTK_OBJECT(label->eventBox), "button_press_event",
-                       GTK_SIGNAL_FUNC(widgetLClickCommandCallback),
-                       (int *)(label->widget->command));
-                       
   gtk_signal_connect(GTK_OBJECT(label->eventBox), "button_press_event",
-                     GTK_SIGNAL_FUNC(widgetMClickCommandCallback),
-                     (int *)(window->middle_click_action));
-  gtk_signal_connect(GTK_OBJECT(label->eventBox), "button_press_event",
-                     GTK_SIGNAL_FUNC(widgetRClickCommandCallback),
-                     (int *)(window->right_click_action));
+                     GTK_SIGNAL_FUNC(widgetLClickCommandCallback),
+                     (int *)(label->widget->command));                       
 
   if (window->background != NULL)
     {
@@ -720,8 +720,8 @@ void generateMenu(ThemeWindow *window)
       gtk_signal_connect(GTK_OBJECT(menuItem->widget),
                          "activate",
                          GTK_SIGNAL_FUNC(widgetCommandCallback),
-                         (int *)(menuItem->command));
-      
+                         (int *)(menuItem->command));                         
+
       menuItems = g_list_next(menuItems);
     }
   while (menuItems != NULL);
@@ -739,15 +739,20 @@ void generateButtons(ThemeWindow *window)
       convertPos(window, button, &x, &y);
       gtk_fixed_put(GTK_FIXED(window->fixed), button->widget, x, y);
       gtk_widget_show(button->widget);
-
-      if (button->command != CMD_POPUP_MENU)
-        gtk_signal_connect(GTK_OBJECT(button->widget), "clicked",
-                           GTK_SIGNAL_FUNC(widgetCommandCallback),
-                           (int *)(button->command));
-      else
+      attachShapeData(button->widget, window);
+      
+      if (button->command == CMD_POPUP_MENU)
         gtk_signal_connect(GTK_OBJECT(button->widget), "button_release_event",
                            GTK_SIGNAL_FUNC(widgetLClickCommandCallback),
                            (int *)(button->command));                           
+      else if (button->command == CMD_DRAG_CAPTURE)
+        gtk_signal_connect(GTK_OBJECT(button->widget), "button_press_event",
+                           GTK_SIGNAL_FUNC(widgetLClickCommandCallback),
+                           (int *)(button->command));
+      else
+        gtk_signal_connect(GTK_OBJECT(button->widget), "clicked",
+                           GTK_SIGNAL_FUNC(widgetCommandCallback),
+                           (int *)(button->command));
 
       buttons = g_list_next(buttons);
     }
@@ -767,8 +772,6 @@ void parseWindow(ThemeWindow *window)
   RcTokens tokens;
   gint height = 0, width = 0;
   gint x = 0, y = 0;
-  gint dragbar_x=0, dragbar_y=0, 
-       dragbar_width=0, dragbar_height=0;
 
   initializeWindow(window);
   tokens.words[0][0] = '\0';  
@@ -789,13 +792,6 @@ void parseWindow(ThemeWindow *window)
         }
       else if (strcasecmp(tokens.words[0], "tooltip") == 0)
         grabQuotedString(tooltip, &tokens);          
-      else if (strcasecmp(tokens.words[0], "dragbar") == 0)
-        {
-          sscanf(tokens.words[1], "%i", &dragbar_x);
-          sscanf(tokens.words[2], "%i", &dragbar_y);
-          sscanf(tokens.words[3], "%i", &dragbar_width);
-          sscanf(tokens.words[4], "%i", &dragbar_height);
-        }  
       else if (strcasecmp(tokens.words[0], "titlebarimage") == 0)          
         strcpy(titlePixmap, tokens.words[1]);
       else if (strcasecmp(tokens.words[0], "background") == 0)
@@ -830,20 +826,20 @@ void parseWindow(ThemeWindow *window)
           else if (strcasecmp(tokens.words[1], "time_label") == 0)
             parseLabel(&(window->time_label));
           else if (strcasecmp(tokens.words[1], "repeat_label") == 0)
-            parseLabel(&(window->repeat_label));
+          parseLabel(&(window->repeat_label)); 
           else if (strcasecmp(tokens.words[1], "menu") == 0)
-            parseMenu(window);
+            parseMenu(window); 
           else if (strcasecmp(tokens.words[1], "volume_slider") == 0)
             parseVolumeSlider(window);
           else if (strcasecmp(tokens.words[1], "button") == 0)
             parseButton(window);
           else
-            rcFileError("invalid identifier");
+            rcFileError("invalid identifier"); 
         }
       else if (strcasecmp(tokens.words[0], "end") == 0)
         continue;
       else
-        rcFileError("invalid window flag");
+        rcFileError("invalid window flag"); 
     }
   
   if (mask)
@@ -856,11 +852,12 @@ void parseWindow(ThemeWindow *window)
       gtk_pixmap_get(GTK_PIXMAP(pixmapWid), &pixmap, &bitmap);                          
       gdk_window_shape_combine_mask(window->main_window->window,
                                     bitmap, 0, 0);
-    }
+    } 
                      
   window->fixed = gtk_fixed_new();
   gtk_container_add(GTK_CONTAINER(window->main_window), window->fixed);
-  gtk_widget_show(window->fixed);  
+  gtk_widget_set_usize(window->fixed, width, height);
+  gtk_widget_show(window->fixed);    
   gtk_widget_realize(window->fixed);
                      
   if (window->background)
@@ -871,31 +868,31 @@ void parseWindow(ThemeWindow *window)
                              NULL, NULL, TRUE);
 
       gtk_pixmap_get(GTK_PIXMAP(pixmapWid), &pixmap, &bitmap);
-      gdk_window_set_back_pixmap(window->fixed->window, pixmap, FALSE);
+      gdk_window_set_back_pixmap(window->fixed->window, pixmap, FALSE); 
     }
-    
+  
   eventBox = gtk_event_box_new();   
-  gtk_widget_set_usize(eventBox, dragbar_width, dragbar_height);
-  gtk_fixed_put(GTK_FIXED(window->fixed), eventBox, dragbar_x, dragbar_y);
-  gtk_widget_show(eventBox);
+  
+  gtk_widget_set_usize(eventBox, width, height);
+  gtk_fixed_put(GTK_FIXED(window->fixed), eventBox, 0, 0);
+  gtk_widget_show(eventBox);  
+  
+#ifdef __DEBUG__
+  g_print("Window Size: %i %i\n", width, height);
+#endif
       
   gdk_window_set_back_pixmap(eventBox->window, pixmap, TRUE);
                      
   window->clickPos = g_new(WidgetLoc, 1);
-  gtk_object_set_user_data(GTK_OBJECT(eventBox), window->clickPos);
-
-  gtk_signal_connect(GTK_OBJECT(eventBox), "button_press_event",
-                     GTK_SIGNAL_FUNC(shape_pressed), window);
-  gtk_signal_connect(GTK_OBJECT(eventBox), "button_release_event",
-                     GTK_SIGNAL_FUNC(shape_released), window);
-  gtk_signal_connect(GTK_OBJECT(eventBox), "motion_notify_event",
-                     GTK_SIGNAL_FUNC(shape_motion), window);             
+  gtk_object_set_user_data(GTK_OBJECT(window->fixed), window->clickPos);
 
   if (! options.managed)
     gdk_window_set_cursor(window->main_window->window, gdk_cursor_new(GDK_LEFT_PTR));
                                  
   gtk_widget_set_usize(window->fixed, width, height);
   gtk_widget_set_uposition(window->main_window, x, y);
+  window->position.x = x;
+  window->position.y = y;
 
   if (window->buttons) generateButtons(window);
   generateLabels(window);  	
@@ -906,13 +903,22 @@ void parseWindow(ThemeWindow *window)
       convertPos(window, &(window->volume_slider), &x, &y);
       gtk_fixed_put(GTK_FIXED(window->fixed), window->volume_slider.widget, x, y);
     }
-    
+
+  gtk_signal_connect(GTK_OBJECT(window->fixed), "button_release_event",
+                     GTK_SIGNAL_FUNC(shape_released), window);
+  gtk_signal_connect(GTK_OBJECT(window->fixed), "motion_notify_event",
+                     GTK_SIGNAL_FUNC(shape_motion), window);             
+
+  gtk_signal_connect(GTK_OBJECT(window->fixed), "button_press_event",
+                     GTK_SIGNAL_FUNC(widgetLClickCommandCallback),
+                     (int *)(window->left_click_action));    
   gtk_signal_connect(GTK_OBJECT(window->fixed), "button_press_event",
                      GTK_SIGNAL_FUNC(widgetMClickCommandCallback),
                      (int *)(window->middle_click_action));
   gtk_signal_connect(GTK_OBJECT(window->fixed), "button_press_event",
                      GTK_SIGNAL_FUNC(widgetRClickCommandCallback),
                      (int *)(window->right_click_action));
+                     
   gtk_signal_connect(GTK_OBJECT(window->main_window), "delete_event",
                      GTK_SIGNAL_FUNC(quitCallback), NULL);                                          
     
@@ -945,8 +951,8 @@ gint parseThemeRC(ThemeWindow *bigWindow,
         parseWindow(bigWindow);
       else if (strcasecmp(tokens.words[1], "small_window") == 0)
         parseWindow(smallWindow);
-      else
-        rcFileError("invalid window identifier");
+      /*      else
+              rcFileError("invalid window identifier"); */
     }
     
   return 0;
